#ifndef _BOARD_H_
#define _BOARD_H_


#include "pattern.h"


#define TRUE  1
#define FALSE 0

enum STONE {
	S_EMPTY,	/* 空点 */
	S_BLACK,	/* 黒　 */
	S_WHITE,	/* 白　 */
	S_OB,		/* 盤外 */
	S_MAX,
};

#define OTHER_COLOR( color ) ((color)^0x3)	/* 色反転 */

#define PASS   0
#define RESIGN -1

#define KOMI 6.5	/* コミ */

#define PURE_BOARD_SIZE	9	/* 盤の本当の大きさ */
#define OB_SIZE 2			/* 盤外の大きさ */
#define BOARD_SIZE	(PURE_BOARD_SIZE + (OB_SIZE + OB_SIZE)) /* 盤の大きさ */

#define BOARD_START OB_SIZE
#define BOARD_END	(OB_SIZE+PURE_BOARD_SIZE-1)

#define PURE_BOARD_MAX	(PURE_BOARD_SIZE * PURE_BOARD_SIZE)
#define BOARD_MAX		(BOARD_SIZE * BOARD_SIZE)

#define MAX_STRINGS (PURE_BOARD_SIZE * PURE_BOARD_SIZE * 4/5)	/* 連の最大数 */

#define MAX_LIBERTIES 3
#define MAX_NEIGHBORS MAX_STRINGS

#define MAX_RECORDS 1000
#define MAX_MOVES MAX_RECORDS-1	/* 最大手数 */

#define POS(x, y)     ((x) + (y) * BOARD_SIZE)
#define X(pos)        ((pos) % BOARD_SIZE)
#define Y(pos)        ((pos) / BOARD_SIZE)
#define NORTH(pos)    ((pos) - BOARD_SIZE)
#define EAST(pos)     ((pos) + 1)
#define SOUTH(pos)    ((pos) + BOARD_SIZE)
#define WEST(pos)     ((pos) - 1)

/* 近傍の座標 */
#define NE  (-BOARD_SIZE+1)
#define NW  (-BOARD_SIZE-1)
#define SE  (BOARD_SIZE+1)
#define SW  (BOARD_SIZE-1)

#define NN  (-BOARD_SIZE-BOARD_SIZE)
#define SS  (BOARD_SIZE+BOARD_SIZE)
#define EE  2
#define WW  (-2)

/* 着手の距離 */
#define DX(pos1,pos2)	(abs(board_x[pos1]-board_x[pos2]))
#define DY(pos1,pos2)	(abs(board_y[pos1]-board_y[pos2]))
#define DIS(pos1,pos2)	(move_dis[DX(pos1,pos2)][DY(pos1,pos2)])


/*------------------------------------------------------------------*/
/* 構造体                                                           */
/*------------------------------------------------------------------*/

/* 着手 */
struct move {
	int color;
	int pos;
};

/* 石 */
struct stone {
	int color;
	int string_number;
	int next_stone;
	int mark;
};

/* 連 */
struct string {
	int color;
	int size;
	int origin;
	int liberties;
	int neighbors;
	int mark;
};

/* ダメ */
struct string_liberties {
	int list[4];
};

/* 隣接する敵の連 */
struct string_neighbors {
	int list[MAX_NEIGHBORS];
};

/* 対局の全情報 */
struct game {
	int prisoner[S_MAX];

	int moves;				
	struct move record[MAX_RECORDS];

	int ko_pos;
	int ko_move;

	struct stone board[BOARD_MAX];

	struct string string[MAX_STRINGS];
	struct string_liberties string_liberties[MAX_STRINGS];
	struct string_neighbors string_neighbors[MAX_STRINGS];
	int next_string;
	int liberty_mark;
	int string_mark;

	int pat3[BOARD_MAX];
	int dia[BOARD_MAX];

	int candidates[BOARD_MAX]; 

	int rate[BOARD_MAX];
	int sum_rate_column[BOARD_SIZE];
	int sum_rate;
};


/*-------------------------------------------------------------------*/
/* グローバル変数                                                    */
/*-------------------------------------------------------------------*/

/* コミ */
extern double komi[3];

/* 盤上の座標 */
extern int onboard_pos[PURE_BOARD_MAX];

/* X座標 */
extern int board_x[BOARD_MAX];

/* Y座標 */
extern int board_y[BOARD_MAX];

/* 4近傍の空点数 */
extern int nb4_empty[PATTERN3_MAX];

/* 眼 */
extern unsigned char eye[PATTERN3_MAX];

/* 距離  */
extern unsigned char border_dis_x[BOARD_MAX];
extern unsigned char border_dis_y[BOARD_MAX];
extern unsigned char move_dis[PURE_BOARD_SIZE][PURE_BOARD_SIZE];;


/*-------------------------------------------------------------------*/
/* 関数                                                              */
/*-------------------------------------------------------------------*/

/* 初期化 (メモリ確保 + 初期設定) */
struct game *InitializeGame( void );

/* メモリ確保 */
struct game *SetupGame( void );

/* 初期設定 */
void ClearGame( struct game *game );

/* コピー */
void CopyGame( struct game *g2, struct game *g1 );

/* メモリ解放 */
void DoneGame( struct game *game );

/* 定数の設定 */
void BoardConst( void );

/* 合法手の判定*/
int CheckLegal( struct game *game, int color, int pos );

/* 合法手かつ眼でない */
int CheckLegalAndEye( struct game *game, int color, int pos );

/* 碁盤に石を置く*/
void SetStone( struct game *game, int color, int pos );

/* 勝敗の判定 */
int BoardScore( struct game *game );

/* 碁盤を表示 */
void DisplayBoard( struct game *game  );


#endif  /* _BOARD_H_ */